/*:
 * @target MZ
 * @plugindesc Event Disappear and Respawn Plugin for MZ - Version 1.0 🌀
 * @author Cody
 *
 * @help
 * This plugin makes an event disappear after interacting and respawn after a set time.
 *
 * Usage:
 * Use the following command in an event's script call:
 *   DisappearAndRespawn(this.eventId(), 60);
 * 
 * - eventId: The ID of the event to disappear and respawn.
 * - respawnTime: The time in seconds for the event to reappear.
 */

(() => {
  
  // Main function to make the event disappear and respawn
  window.DisappearAndRespawn = function(eventId, respawnTime) {
    if (!eventId || !respawnTime) return;

    const mapId = $gameMap.mapId();
    const key = [mapId, eventId, 'A']; // Self Switch A is used to control visibility

    // Set the event's self switch to true to make it disappear
    $gameSelfSwitches.setValue(key, true);

    // Set a timeout to make the event reappear after the specified time
    setTimeout(() => {
      $gameSelfSwitches.setValue(key, false);
    }, respawnTime * 1000); // Convert seconds to milliseconds
  };

  // Example usage in event: DisappearAndRespawn(this.eventId(), 60);

})();
